@@ -12,8 +12,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -35,12 +35,12 @@ assistant_view_open_uri_cb (DhAssistantView *view,
                             DhAssistant     *assistant)
 {
         DhApp *app;
-        GtkWindow *window;
+        DhWindow *window;
 
         app = DH_APP (gtk_window_get_application (GTK_WINDOW (assistant)));
 
-        window = dh_app_peek_first_window (app);
-        _dh_window_display_uri (DH_WINDOW (window), uri);
+        window = dh_app_get_active_main_window (app, TRUE);
+        _dh_window_display_uri (window, uri);
 }
 
 static gboolean
@@ -64,8 +64,7 @@ dh_assistant_configure_event (GtkWidget         *widget,
         settings = dh_settings_get_singleton ();
 
         dh_util_window_settings_save (GTK_WINDOW (widget),
-                                      dh_settings_peek_assistant_settings (settings),
-                                      FALSE);
+                                      dh_settings_peek_assistant_settings (settings));
 
         return GTK_WIDGET_CLASS (dh_assistant_parent_class)->configure_event (widget, event);
 }
@@ -88,27 +87,30 @@ static void
 dh_assistant_init (DhAssistant *assistant)
 {
         DhAssistantPrivate *priv = dh_assistant_get_instance_private (assistant);
-        DhSettings *settings;
 
         gtk_widget_init_template (GTK_WIDGET (assistant));
 
         g_signal_connect (priv->view, "open-uri",
                           G_CALLBACK (assistant_view_open_uri_cb),
                           assistant);
-
-        settings = dh_settings_get_singleton ();
-
-        dh_util_window_settings_restore (GTK_WINDOW (assistant),
-                                         dh_settings_peek_assistant_settings (settings),
-                                         FALSE);
 }
 
-GtkWidget *
+DhAssistant *
 dh_assistant_new (DhApp *application)
 {
-        return g_object_new (DH_TYPE_ASSISTANT,
-                             "application", application,
-                             NULL);
+        DhAssistant *assistant;
+        DhSettings *settings;
+
+        assistant = g_object_new (DH_TYPE_ASSISTANT,
+                                  "application", application,
+                                  NULL);
+
+        settings = dh_settings_get_singleton ();
+        gtk_widget_realize (GTK_WIDGET (assistant));
+        dh_util_window_settings_restore (GTK_WINDOW (assistant),
+                                         dh_settings_peek_assistant_settings (settings));
+
+        return assistant;
 }
 
 gboolean
