@@ -4,6 +4,7 @@
  * Copyright (C) 2002 Mikael Hallendal <micke@imendio.com>
  * Copyright (C) 2004-2008 Imendio AB
  * Copyright (C) 2010 Lanedo GmbH
+ * Copyright (C) 2017, 2018 Sébastien Wilmet <swilmet@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,9 +33,15 @@
  * @Short_description: A book, usually the documentation for one library
  *
  * A #DhBook usually contains the documentation for one library (or
- * application), for example GLib or GTK+. There is one #DhBook for each index
- * file found and parsed (an index file is a file with the extension `*.devhelp`
- * or `*.devhelp2`).
+ * application), for example GLib or GTK+. A #DhBook corresponds to one index
+ * file. An index file is a file with the extension `*.devhelp`, `*.devhelp2`,
+ * `*.devhelp.gz` or `*.devhelp2.gz`.
+ *
+ * #DhBook creates a #GFileMonitor on the index file, and emits the
+ * #DhBook::updated or #DhBook::deleted signal in case the index file has
+ * changed on the filesystem. #DhBookManager listens to those #DhBook signals,
+ * and emits in turn the #DhBookManager::book-deleted and
+ * #DhBookManager::book-created signals.
  */
 
 /* Timeout to wait for new events on the index file so that they are merged and
@@ -43,10 +50,16 @@
 #define EVENT_MERGE_TIMEOUT_SECS (2)
 
 enum {
-        SIGNAL_BOOK_ENABLED,
-        SIGNAL_BOOK_DISABLED,
-        SIGNAL_BOOK_UPDATED,
-        SIGNAL_BOOK_DELETED,
+        /* FIXME: a boolean property would be a better API instead of the
+         * ::enabled and ::disabled signals. Or this whole concept can be
+         * removed from DhBook, by introducing DhBookSelection, see:
+         * https://bugzilla.gnome.org/show_bug.cgi?id=784491#c3
+         */
+        SIGNAL_ENABLED,
+        SIGNAL_DISABLED,
+
+        SIGNAL_UPDATED,
+        SIGNAL_DELETED,
         N_SIGNALS
 };
 
@@ -69,8 +82,7 @@ typedef struct {
         /* List of DhLink*. */
         GList *links;
 
-        /* Generated list of completions (gchar*) in the book. */
-        GList *completions;
+        DhCompletion *completion;
 
         GFileMonitor *index_file_monitor;
         BookMonitorEvent last_monitor_event;
@@ -90,6 +102,7 @@ dh_book_dispose (GObject *object)
 
         priv = dh_book_get_instance_private (DH_BOOK (object));
 
+        g_clear_object (&priv->completion);
         g_clear_object (&priv->index_file_monitor);
 
         if (priv->monitor_event_timeout_id != 0) {
@@ -113,7 +126,6 @@ dh_book_finalize (GObject *object)
         g_free (priv->language);
         _dh_util_free_book_tree (priv->tree);
         g_list_free_full (priv->links, (GDestroyNotify)dh_link_unref);
-        g_list_free_full (priv->completions, g_free);
 
         G_OBJECT_CLASS (dh_book_parent_class)->finalize (object);
 }
@@ -130,7 +142,7 @@ dh_book_class_init (DhBookClass *klass)
          * DhBook::enabled:
          * @book: the #DhBook emitting the signal.
          */
-        signals[SIGNAL_BOOK_ENABLED] =
+        signals[SIGNAL_ENABLED] =
                 g_signal_new ("enabled",
                               G_TYPE_FROM_CLASS (klass),
                               G_SIGNAL_RUN_LAST,
@@ -143,7 +155,7 @@ dh_book_class_init (DhBookClass *klass)
          * DhBook::disabled:
          * @book: the #DhBook emitting the signal.
          */
-        signals[SIGNAL_BOOK_DISABLED] =
+        signals[SIGNAL_DISABLED] =
                 g_signal_new ("disabled",
                               G_TYPE_FROM_CLASS (klass),
                               G_SIGNAL_RUN_LAST,
@@ -155,8 +167,11 @@ dh_book_class_init (DhBookClass *klass)
         /**
          * DhBook::updated:
          * @book: the #DhBook emitting the signal.
+         *
+         * The ::updated signal is emitted when the index file has been
+         * modified (but the file still exists).
          */
-        signals[SIGNAL_BOOK_UPDATED] =
+        signals[SIGNAL_UPDATED] =
                 g_signal_new ("updated",
                               G_TYPE_FROM_CLASS (klass),
                               G_SIGNAL_RUN_LAST,
@@ -168,8 +183,11 @@ dh_book_class_init (DhBookClass *klass)
         /**
          * DhBook::deleted:
          * @book: the #DhBook emitting the signal.
+         *
+         * The ::deleted signal is emitted when the index file has been deleted
+         * from the filesystem.
          */
-        signals[SIGNAL_BOOK_DELETED] =
+        signals[SIGNAL_DELETED] =
                 g_signal_new ("deleted",
                               G_TYPE_FROM_CLASS (klass),
                               G_SIGNAL_RUN_LAST,
@@ -209,7 +227,7 @@ monitor_event_timeout_cb (gpointer data)
                  * doing it.
                  */
                 g_object_ref (book);
-                g_signal_emit (book, signals[SIGNAL_BOOK_DELETED], 0);
+                g_signal_emit (book, signals[SIGNAL_DELETED], 0);
                 g_object_unref (book);
                 break;
 
@@ -218,7 +236,7 @@ monitor_event_timeout_cb (gpointer data)
                  * doing it.
                  */
                 g_object_ref (book);
-                g_signal_emit (book, signals[SIGNAL_BOOK_UPDATED], 0);
+                g_signal_emit (book, signals[SIGNAL_UPDATED], 0);
                 g_object_unref (book);
                 break;
 
@@ -294,7 +312,12 @@ dh_book_new (GFile *index_file)
                                   &priv->tree,
                                   &priv->links,
                                   &error)) {
-                if (error != NULL) {
+                /* It's fine if the file doesn't exist, as DhBookManager tries
+                 * to create a DhBook for each possible index file in a certain
+                 * book directory.
+                 */
+                if (error != NULL &&
+                    !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND)) {
                         gchar *parse_name;
 
                         parse_name = g_file_get_parse_name (priv->index_file);
@@ -304,9 +327,10 @@ dh_book_new (GFile *index_file)
                                    error->message);
 
                         g_free (parse_name);
-                        g_clear_error (&error);
                 }
 
+                g_clear_error (&error);
+
                 /* Deallocate the book, as we are not going to add it in the
                  * manager.
                  */
@@ -316,8 +340,8 @@ dh_book_new (GFile *index_file)
 
         /* Rewrite language, if any, including the prefix we want to use when
          * seeing it, to standarize how the language group is shown.
-         * FIXME: do this in DhLanguage instead (this may require to implement
-         * DhLanguage differently).
+         * FIXME: maybe instead of a string, have a DhLanguage object which
+         * canonicalizes the string.
          */
         dh_util_ascii_strtitle (language);
         priv->language = (language != NULL ?
@@ -357,15 +381,13 @@ dh_book_new (GFile *index_file)
 }
 
 /**
- * dh_book_get_links:
+ * dh_book_get_index_file:
  * @book: a #DhBook.
  *
- * Returns: (element-type DhLink) (transfer none) (nullable): the list of
- * <emphasis>all</emphasis> #DhLink's part of @book, or %NULL if the book is
- * disabled.
+ * Returns: (transfer none): the index file.
  */
-GList *
-dh_book_get_links (DhBook *book)
+GFile *
+dh_book_get_index_file (DhBook *book)
 {
         DhBookPrivate *priv;
 
@@ -373,76 +395,7 @@ dh_book_get_links (DhBook *book)
 
         priv = dh_book_get_instance_private (book);
 
-        return priv->enabled ? priv->links : NULL;
-}
-
-/**
- * dh_book_get_completions:
- * @book: a #DhBook.
- *
- * Returns: (element-type utf8) (transfer none) (nullable): the completions
- * associated with the book.
- */
-GList *
-dh_book_get_completions (DhBook *book)
-{
-        DhBookPrivate *priv;
-
-        g_return_val_if_fail (DH_IS_BOOK (book), NULL);
-
-        priv = dh_book_get_instance_private (book);
-
-        if (!priv->enabled)
-                return NULL;
-
-        if (priv->completions == NULL) {
-                GList *l;
-
-                for (l = priv->links; l != NULL; l = l->next) {
-                        DhLink *link = l->data;
-                        gchar *str;
-
-                        /* Add additional "page:" and "book:" completions. */
-                        if (dh_link_get_link_type (link) == DH_LINK_TYPE_BOOK) {
-                                str = g_strdup_printf ("book:%s", dh_link_get_name (link));
-                                priv->completions = g_list_prepend (priv->completions, str);
-                        }
-                        else if (dh_link_get_link_type (link) == DH_LINK_TYPE_PAGE) {
-                                str = g_strdup_printf ("page:%s", dh_link_get_name (link));
-                                priv->completions = g_list_prepend (priv->completions, str);
-                        }
-
-                        str = g_strdup (dh_link_get_name (link));
-                        priv->completions = g_list_prepend (priv->completions, str);
-                }
-        }
-
-        return priv->completions;
-}
-
-/**
- * dh_book_get_tree:
- * @book: a #DhBook.
- *
- * Gets the general structure of the book, as a tree. The tree contains only
- * #DhLink's of type %DH_LINK_TYPE_BOOK or %DH_LINK_TYPE_PAGE. The other
- * #DhLink's are not contained in the tree. To have a list of
- * <emphasis>all</emphasis> #DhLink's part of the book, you need to call
- * dh_book_get_links().
- *
- * Returns: (transfer none) (nullable): the tree of #DhLink's part of the @book,
- * or %NULL if the book is disabled.
- */
-GNode *
-dh_book_get_tree (DhBook *book)
-{
-        DhBookPrivate *priv;
-
-        g_return_val_if_fail (DH_IS_BOOK (book), NULL);
-
-        priv = dh_book_get_instance_private (book);
-
-        return priv->enabled ? priv->tree : NULL;
+        return priv->index_file;
 }
 
 /**
@@ -489,7 +442,7 @@ dh_book_get_title (DhBook *book)
  * dh_book_get_language:
  * @book: a #DhBook.
  *
- * Returns: the book language.
+ * Returns: the programming language used in @book.
  */
 const gchar *
 dh_book_get_language (DhBook *book)
@@ -504,13 +457,15 @@ dh_book_get_language (DhBook *book)
 }
 
 /**
- * dh_book_get_index_file:
+ * dh_book_get_links:
  * @book: a #DhBook.
  *
- * Returns: (transfer none): the index file.
+ * Returns: (element-type DhLink) (transfer none) (nullable): the list of
+ * <emphasis>all</emphasis> #DhLink's part of @book, or %NULL if the book is
+ * disabled.
  */
-GFile *
-dh_book_get_index_file (DhBook *book)
+GList *
+dh_book_get_links (DhBook *book)
 {
         DhBookPrivate *priv;
 
@@ -518,7 +473,67 @@ dh_book_get_index_file (DhBook *book)
 
         priv = dh_book_get_instance_private (book);
 
-        return priv->index_file;
+        return priv->enabled ? priv->links : NULL;
+}
+
+/**
+ * dh_book_get_tree:
+ * @book: a #DhBook.
+ *
+ * Gets the general structure of the book, as a tree. The tree contains only
+ * #DhLink's of type %DH_LINK_TYPE_BOOK or %DH_LINK_TYPE_PAGE. The other
+ * #DhLink's are not contained in the tree. To have a list of
+ * <emphasis>all</emphasis> #DhLink's part of the book, you need to call
+ * dh_book_get_links().
+ *
+ * Returns: (transfer none) (nullable): the tree of #DhLink's part of the @book,
+ * or %NULL if the book is disabled.
+ */
+GNode *
+dh_book_get_tree (DhBook *book)
+{
+        DhBookPrivate *priv;
+
+        g_return_val_if_fail (DH_IS_BOOK (book), NULL);
+
+        priv = dh_book_get_instance_private (book);
+
+        return priv->enabled ? priv->tree : NULL;
+}
+
+/**
+ * dh_book_get_completion:
+ * @book: a #DhBook.
+ *
+ * Returns: (transfer none): the #DhCompletion of @book.
+ * Since: 3.28
+ */
+DhCompletion *
+dh_book_get_completion (DhBook *book)
+{
+        DhBookPrivate *priv;
+
+        g_return_val_if_fail (DH_IS_BOOK (book), NULL);
+
+        priv = dh_book_get_instance_private (book);
+
+        if (priv->completion == NULL) {
+                GList *l;
+
+                priv->completion = dh_completion_new ();
+
+                for (l = priv->links; l != NULL; l = l->next) {
+                        DhLink *link = l->data;
+                        const gchar *str;
+
+                        str = dh_link_get_name (link);
+                        dh_completion_add_string (priv->completion, str);
+                }
+
+                dh_completion_sort (priv->completion);
+        }
+
+        return priv->completion;
 }
 
 /**
@@ -558,50 +573,23 @@ dh_book_set_enabled (DhBook   *book,
 
         enabled = enabled != FALSE;
 
+        /* Create DhCompletion, because if all the DhCompletion objects need to
+         * be created (synchronously) at the time of the first completion, it
+         * can make the GUI not responsive (measured time was for example 40ms
+         * to create the DhCompletion's for 17 books, which is not a lot of
+         * books). On application startup it is less a problem.
+         */
+        if (enabled)
+                dh_book_get_completion (book);
+
         if (priv->enabled != enabled) {
                 priv->enabled = enabled;
                 g_signal_emit (book,
-                               enabled ? signals[SIGNAL_BOOK_ENABLED] : signals[SIGNAL_BOOK_DISABLED],
+                               enabled ? signals[SIGNAL_ENABLED] : signals[SIGNAL_DISABLED],
                                0);
         }
 }
 
-/**
- * dh_book_cmp_by_path:
- * @a: a #DhBook.
- * @b: a #DhBook.
- *
- * Compares the #DhBook's by their path to the index file.
- *
- * Returns: an integer less than, equal to, or greater than zero, if @a is <, ==
- * or > than @b.
- */
-gint
-dh_book_cmp_by_path (DhBook *a,
-                     DhBook *b)
-{
-        DhBookPrivate *priv_a;
-        DhBookPrivate *priv_b;
-        gchar *path_a;
-        gchar *path_b;
-        gint diff;
-
-        if (a == NULL || b == NULL)
-                return -1;
-
-        priv_a = dh_book_get_instance_private (a);
-        priv_b = dh_book_get_instance_private (b);
-
-        path_a = g_file_get_path (priv_a->index_file);
-        path_b = g_file_get_path (priv_b->index_file);
-
-        diff = g_strcmp0 (path_a, path_b);
-
-        g_free (path_a);
-        g_free (path_b);
-        return diff;
-}
-
 /**
  * dh_book_cmp_by_id:
  * @a: a #DhBook.
