@@ -5,7 +5,7 @@
  * Copyright (C) 2005-2008 Imendio AB
  * Copyright (C) 2010 Lanedo GmbH
  * Copyright (C) 2013 Aleksander Morgado <aleksander@gnu.org>
- * Copyright (C) 2015 Sébastien Wilmet <swilmet@gnome.org>
+ * Copyright (C) 2015, 2017, 2018 Sébastien Wilmet <swilmet@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,18 +17,14 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "config.h"
 #include "dh-sidebar.h"
-
-#include <string.h>
-
-#include "dh-keyword-model.h"
 #include "dh-book.h"
 #include "dh-book-tree.h"
+#include "dh-keyword-model.h"
 
 /**
  * SECTION:dh-sidebar
@@ -56,46 +52,88 @@
  */
 
 typedef struct {
-        DhBookTree              *book_tree;
-        GtkScrolledWindow       *sw_book_tree;
+        /* A GtkSearchEntry. */
+        GtkEntry *entry;
 
-        GtkEntry                *entry;
-        DhKeywordModel          *hitlist_model;
-        GtkTreeView             *hitlist_view;
-        GtkScrolledWindow       *sw_hitlist;
+        DhBookTree *book_tree;
+        GtkScrolledWindow *sw_book_tree;
 
-        GCompletion             *completion;
-        guint                    idle_complete_id;
-        guint                    idle_filter_id;
+        DhKeywordModel *hitlist_model;
+        GtkTreeView *hitlist_view;
+        GtkScrolledWindow *sw_hitlist;
+
+        guint idle_complete_id;
+        guint idle_search_id;
 } DhSidebarPrivate;
 
 enum {
-        LINK_SELECTED,
+        SIGNAL_LINK_SELECTED,
         N_SIGNALS
 };
 
 static guint signals[N_SIGNALS] = { 0 };
 
-G_DEFINE_TYPE_WITH_PRIVATE (DhSidebar, dh_sidebar, GTK_TYPE_BOX)
+G_DEFINE_TYPE_WITH_PRIVATE (DhSidebar, dh_sidebar, GTK_TYPE_GRID)
+
+static void
+dh_sidebar_dispose (GObject *object)
+{
+        DhSidebarPrivate *priv = dh_sidebar_get_instance_private (DH_SIDEBAR (object));
+
+        g_clear_object (&priv->hitlist_model);
+
+        if (priv->idle_complete_id != 0) {
+                g_source_remove (priv->idle_complete_id);
+                priv->idle_complete_id = 0;
+        }
+
+        if (priv->idle_search_id != 0) {
+                g_source_remove (priv->idle_search_id);
+                priv->idle_search_id = 0;
+        }
+
+        G_OBJECT_CLASS (dh_sidebar_parent_class)->dispose (object);
+}
+
+static void
+dh_sidebar_class_init (DhSidebarClass *klass)
+{
+        GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+        object_class->dispose = dh_sidebar_dispose;
+
+        /**
+         * DhSidebar::link-selected:
+         * @sidebar: a #DhSidebar.
+         * @link: the selected #DhLink.
+         */
+        signals[SIGNAL_LINK_SELECTED] =
+                g_signal_new ("link-selected",
+                              G_TYPE_FROM_CLASS (klass),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (DhSidebarClass, link_selected),
+                              NULL, NULL, NULL,
+                              G_TYPE_NONE,
+                              1, DH_TYPE_LINK);
+}
 
 /******************************************************************************/
 
 static gboolean
-sidebar_filter_idle_cb (DhSidebar *sidebar)
+search_idle_cb (gpointer user_data)
 {
-        DhSidebarPrivate *priv;
+        DhSidebar *sidebar = DH_SIDEBAR (user_data);
+        DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
         const gchar *search_text;
         const gchar *book_id;
-        DhLink      *book_link;
-        DhLink      *link;
+        DhLink *book_link;
+        DhLink *exact_link;
 
-        priv = dh_sidebar_get_instance_private (sidebar);
-
-        priv->idle_filter_id = 0;
+        priv->idle_search_id = 0;
 
         search_text = gtk_entry_get_text (priv->entry);
 
-        book_link = dh_sidebar_get_selected_book (sidebar);
+        book_link = dh_book_tree_get_selected_book (priv->book_tree);
         book_id = book_link != NULL ? dh_link_get_book_id (book_link) : NULL;
 
         /* Disconnect the model during the filter, for:
@@ -104,16 +142,16 @@ sidebar_filter_idle_cb (DhSidebar *sidebar)
          */
         gtk_tree_view_set_model (priv->hitlist_view, NULL);
 
-        link = dh_keyword_model_filter (priv->hitlist_model,
-                                        search_text,
-                                        book_id,
-                                        NULL);
+        exact_link = dh_keyword_model_filter (priv->hitlist_model,
+                                              search_text,
+                                              book_id,
+                                              NULL);
 
         gtk_tree_view_set_model (priv->hitlist_view,
                                  GTK_TREE_MODEL (priv->hitlist_model));
 
-        if (link != NULL)
-                g_signal_emit (sidebar, signals[LINK_SELECTED], 0, link);
+        if (exact_link != NULL)
+                g_signal_emit (sidebar, signals[SIGNAL_LINK_SELECTED], 0, exact_link);
 
         if (book_link != NULL)
                 dh_link_unref (book_link);
@@ -122,88 +160,26 @@ sidebar_filter_idle_cb (DhSidebar *sidebar)
 }
 
 static void
-sidebar_search_run_idle (DhSidebar *sidebar)
+setup_search_idle (DhSidebar *sidebar)
 {
         DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
 
-        if (priv->idle_filter_id == 0)
-                priv->idle_filter_id =
-                        g_idle_add ((GSourceFunc) sidebar_filter_idle_cb, sidebar);
+        if (priv->idle_search_id == 0)
+                priv->idle_search_id = g_idle_add (search_idle_cb, sidebar);
+}
+
+static void
+book_manager_changed_cb (DhSidebar *sidebar)
+{
+        /* Update current search if any. */
+        setup_search_idle (sidebar);
 }
 
 /******************************************************************************/
 
 static void
-sidebar_completion_add_book (DhSidebar *sidebar,
-                             DhBook    *book)
-{
-        GList *completions;
-        DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
-
-        if (G_UNLIKELY (priv->completion == NULL))
-                priv->completion = g_completion_new (NULL);
-
-        completions = dh_book_get_completions (book);
-        if (completions != NULL)
-                g_completion_add_items (priv->completion, completions);
-}
-
-static void
-sidebar_completion_delete_book (DhSidebar *sidebar,
-                                DhBook    *book)
-{
-        GList *completions;
-        DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
-
-        if (G_UNLIKELY (priv->completion == NULL))
-                return;
-
-        completions = dh_book_get_completions (book);
-        if (completions != NULL)
-                g_completion_remove_items (priv->completion, completions);
-}
-
-static void
-sidebar_book_created_or_enabled_cb (DhBookManager *book_manager,
-                                    DhBook        *book,
-                                    DhSidebar     *sidebar)
-{
-        sidebar_completion_add_book (sidebar, book);
-
-        /* Update current search if any */
-        sidebar_search_run_idle (sidebar);
-}
-
-static void
-sidebar_book_deleted_or_disabled_cb (DhBookManager *book_manager,
-                                     DhBook        *book,
-                                     DhSidebar     *sidebar)
-{
-        sidebar_completion_delete_book (sidebar, book);
-
-        /* Update current search if any */
-        sidebar_search_run_idle (sidebar);
-}
-
-static void
-sidebar_completion_populate (DhSidebar *sidebar)
-{
-        DhBookManager *book_manager;
-        GList *books;
-        GList *l;
-
-        book_manager = dh_book_manager_get_singleton ();
-        books = dh_book_manager_get_books (book_manager);
-
-        for (l = books; l != NULL; l = l->next)
-                sidebar_completion_add_book (sidebar, DH_BOOK (l->data));
-}
-
-/******************************************************************************/
-
-static void
-sidebar_hitlist_selection_changed_cb (GtkTreeSelection *selection,
-                                      DhSidebar        *sidebar)
+hitlist_selection_changed_cb (GtkTreeSelection *selection,
+                              DhSidebar        *sidebar)
 {
         DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
         GtkTreeIter iter;
@@ -215,7 +191,7 @@ sidebar_hitlist_selection_changed_cb (GtkTreeSelection *selection,
                                     DH_KEYWORD_MODEL_COL_LINK, &link,
                                     -1);
 
-                g_signal_emit (sidebar, signals[LINK_SELECTED], 0, link);
+                g_signal_emit (sidebar, signals[SIGNAL_LINK_SELECTED], 0, link);
         }
 }
 
@@ -223,14 +199,14 @@ sidebar_hitlist_selection_changed_cb (GtkTreeSelection *selection,
  * html view has been scrolled away.
  */
 static gboolean
-sidebar_hitlist_button_press_cb (GtkTreeView    *hitlist_view,
-                                 GdkEventButton *event,
-                                 DhSidebar      *sidebar)
+hitlist_button_press_cb (GtkTreeView    *hitlist_view,
+                         GdkEventButton *event,
+                         DhSidebar      *sidebar)
 {
         DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
-        GtkTreePath  *path;
-        GtkTreeIter   iter;
-        DhLink       *link;
+        GtkTreePath *path;
+        GtkTreeIter iter;
+        DhLink *link;
 
         gtk_tree_view_get_path_at_pos (hitlist_view, event->x, event->y, &path,
                                        NULL, NULL, NULL);
@@ -245,7 +221,7 @@ sidebar_hitlist_button_press_cb (GtkTreeView    *hitlist_view,
                             DH_KEYWORD_MODEL_COL_LINK, &link,
                             -1);
 
-        g_signal_emit (sidebar, signals[LINK_SELECTED], 0, link);
+        g_signal_emit (sidebar, signals[SIGNAL_LINK_SELECTED], 0, link);
 
         /* Always propagate the event so the tree view can update
          * the selection etc.
@@ -254,9 +230,9 @@ sidebar_hitlist_button_press_cb (GtkTreeView    *hitlist_view,
 }
 
 static gboolean
-sidebar_entry_key_press_event_cb (GtkEntry    *entry,
-                                  GdkEventKey *event,
-                                  DhSidebar   *sidebar)
+entry_key_press_event_cb (GtkEntry    *entry,
+                          GdkEventKey *event,
+                          DhSidebar   *sidebar)
 {
         DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
 
@@ -265,8 +241,8 @@ sidebar_entry_key_press_event_cb (GtkEntry    *entry,
                         if (gtk_widget_is_visible (GTK_WIDGET (priv->hitlist_view)))
                                 gtk_widget_grab_focus (GTK_WIDGET (priv->hitlist_view));
                 } else {
+                        gtk_editable_select_region (GTK_EDITABLE (entry), 0, 0);
                         gtk_editable_set_position (GTK_EDITABLE (entry), -1);
-                        gtk_editable_select_region (GTK_EDITABLE (entry), -1, -1);
                 }
 
                 return GDK_EVENT_STOP;
@@ -274,11 +250,23 @@ sidebar_entry_key_press_event_cb (GtkEntry    *entry,
 
         if (event->keyval == GDK_KEY_Return ||
             event->keyval == GDK_KEY_KP_Enter) {
-                GtkTreeIter  iter;
-                DhLink      *link;
-                gchar       *name;
+                GtkTreeIter iter;
+                DhLink *link;
+                gchar *name;
 
-                /* Get the first entry found. */
+                /* Get the first entry found.
+                 *
+                 * FIXME: is it really useful to do that? If there is an exact
+                 * match it already gets selected, so it seems that the feature
+                 * here just selects a random symbol (the one that appears to be
+                 * the first in the list).
+                 * I've never used this feature -- swilmet.
+                 * This has been implemented in
+                 * commit 455440a93d1b55d5a1e53ecabb2ee33093eec965
+                 * and https://bugzilla.gnome.org/show_bug.cgi?id=114558
+                 * but maybe at that time the search didn't jump to the exact
+                 * match if there was one.
+                 */
                 if (gtk_widget_is_visible (GTK_WIDGET (priv->hitlist_view)) &&
                     gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->hitlist_model), &iter)) {
                         gtk_tree_model_get (GTK_TREE_MODEL (priv->hitlist_model),
@@ -290,10 +278,10 @@ sidebar_entry_key_press_event_cb (GtkEntry    *entry,
                         gtk_entry_set_text (entry, name);
                         g_free (name);
 
+                        gtk_editable_select_region (GTK_EDITABLE (entry), 0, 0);
                         gtk_editable_set_position (GTK_EDITABLE (entry), -1);
-                        gtk_editable_select_region (GTK_EDITABLE (entry), -1, -1);
 
-                        g_signal_emit (sidebar, signals[LINK_SELECTED], 0, link);
+                        g_signal_emit (sidebar, signals[SIGNAL_LINK_SELECTED], 0, link);
 
                         return GDK_EVENT_STOP;
                 }
@@ -303,105 +291,96 @@ sidebar_entry_key_press_event_cb (GtkEntry    *entry,
 }
 
 static void
-sidebar_entry_changed_cb (GtkEntry  *entry,
-                          DhSidebar *sidebar)
+entry_changed_cb (GtkEntry  *entry,
+                  DhSidebar *sidebar)
 {
         DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
         const gchar *search_text;
 
         search_text = gtk_entry_get_text (entry);
 
+        /* We don't want a delay when the search text becomes empty, to show the
+         * book tree. So do it here and not in entry_search_changed_cb().
+         */
         if (search_text == NULL || search_text[0] == '\0') {
                 gtk_widget_hide (GTK_WIDGET (priv->sw_hitlist));
                 gtk_widget_show (GTK_WIDGET (priv->sw_book_tree));
-        } else {
+        }
+}
+
+static void
+entry_search_changed_cb (GtkSearchEntry *search_entry,
+                         DhSidebar      *sidebar)
+{
+        DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
+        const gchar *search_text;
+
+        search_text = gtk_entry_get_text (GTK_ENTRY (search_entry));
+
+        if (search_text != NULL && search_text[0] != '\0') {
                 gtk_widget_hide (GTK_WIDGET (priv->sw_book_tree));
                 gtk_widget_show (GTK_WIDGET (priv->sw_hitlist));
-                sidebar_search_run_idle (sidebar);
+                setup_search_idle (sidebar);
         }
 }
 
 static gboolean
-sidebar_complete_idle_cb (DhSidebar *sidebar)
+complete_idle_cb (gpointer user_data)
 {
+        DhSidebar *sidebar = DH_SIDEBAR (user_data);
         DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
-        const gchar  *search_text;
-        gchar        *completed = NULL;
+        DhBookManager *book_manager;
+        GList *books;
+        GList *l;
+        GList *completion_objects = NULL;
+        const gchar *search_text;
+        gchar *completed;
 
-        search_text = gtk_entry_get_text (priv->entry);
+        book_manager = dh_book_manager_get_singleton ();
+        books = dh_book_manager_get_books (book_manager);
+        for (l = books; l != NULL; l = l->next) {
+                DhBook *cur_book = DH_BOOK (l->data);
 
-        g_completion_complete (priv->completion, search_text, &completed);
-        if (completed != NULL) {
-                gsize length = strlen (search_text);
+                if (dh_book_get_enabled (cur_book)) {
+                        DhCompletion *completion;
 
-                gtk_entry_set_text (priv->entry, completed);
-                gtk_editable_set_position (GTK_EDITABLE (priv->entry), length);
-                gtk_editable_select_region (GTK_EDITABLE (priv->entry),
-                                            length, -1);
-                g_free (completed);
+                        completion = dh_book_get_completion (cur_book);
+                        completion_objects = g_list_prepend (completion_objects, completion);
+                }
         }
 
-        priv->idle_complete_id = 0;
+        search_text = gtk_entry_get_text (priv->entry);
+        completed = dh_completion_aggregate_complete (completion_objects, search_text);
 
+        if (completed != NULL) {
+                guint16 n_chars_before;
+
+                n_chars_before = gtk_entry_get_text_length (priv->entry);
+
+                gtk_entry_set_text (priv->entry, completed);
+                gtk_editable_set_position (GTK_EDITABLE (priv->entry), n_chars_before);
+                gtk_editable_select_region (GTK_EDITABLE (priv->entry),
+                                            n_chars_before, -1);
+        }
+
+        g_list_free (completion_objects);
+        g_free (completed);
+
+        priv->idle_complete_id = 0;
         return G_SOURCE_REMOVE;
 }
 
 static void
-sidebar_entry_insert_text_cb (GtkEntry    *entry,
-                              const gchar *text,
-                              gint         length,
-                              gint        *position,
-                              DhSidebar   *sidebar)
+entry_insert_text_cb (GtkEntry    *entry,
+                      const gchar *text,
+                      gint         length,
+                      gint        *position,
+                      DhSidebar   *sidebar)
 {
         DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
 
         if (priv->idle_complete_id == 0)
-                priv->idle_complete_id =
-                        g_idle_add ((GSourceFunc) sidebar_complete_idle_cb, sidebar);
-}
-
-/**
- * dh_sidebar_set_search_string:
- * @sidebar: a #DhSidebar.
- * @str: the string to search.
- */
-void
-dh_sidebar_set_search_string (DhSidebar   *sidebar,
-                              const gchar *str)
-{
-        DhSidebarPrivate *priv;
-
-        g_return_if_fail (DH_IS_SIDEBAR (sidebar));
-
-        priv = dh_sidebar_get_instance_private (sidebar);
-
-        gtk_entry_set_text (priv->entry, str);
-        gtk_editable_set_position (GTK_EDITABLE (priv->entry), -1);
-        gtk_editable_select_region (GTK_EDITABLE (priv->entry), -1, -1);
-
-        /* If the GtkEntry text was already equal to @str, the
-         * GtkEditable::changed signal was not emitted, so force to emit it to
-         * call sidebar_entry_changed_cb(), forcing a new search. If an exact
-         * match is found, the DhSidebar::link-selected signal will be emitted,
-         * to re-jump to that symbol (even if the GtkEntry text was equal, it
-         * doesn't mean that the WebKitWebView was showing the exact match).
-         * https://bugzilla.gnome.org/show_bug.cgi?id=776596
-         */
-        g_signal_emit_by_name (priv->entry, "changed");
-}
-
-/**
- * dh_sidebar_set_search_focus:
- * @sidebar: a #DhSidebar.
- *
- * Gives the focus to the search entry.
- */
-void
-dh_sidebar_set_search_focus (DhSidebar *sidebar)
-{
-        DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
-
-        gtk_widget_grab_focus (GTK_WIDGET (priv->entry));
+                priv->idle_complete_id = g_idle_add (complete_idle_cb, sidebar);
 }
 
 static void
@@ -411,12 +390,12 @@ hitlist_cell_data_func (GtkTreeViewColumn *tree_column,
                         GtkTreeIter       *iter,
                         gpointer           data)
 {
-        DhLink       *link;
-        DhLinkType    link_type;
-        PangoStyle    style;
-        PangoWeight   weight;
-        gboolean      current_book_flag;
-        gchar        *name;
+        DhLink *link;
+        DhLinkType link_type;
+        PangoStyle style;
+        PangoWeight weight;
+        gboolean current_book_flag;
+        gchar *name;
 
         gtk_tree_model_get (hitlist_model, iter,
                             DH_KEYWORD_MODEL_COL_LINK, &link,
@@ -431,19 +410,21 @@ hitlist_cell_data_func (GtkTreeViewColumn *tree_column,
         /* Matches on the current book are given in bold. Note that we check the
          * current book as it was given to the DhKeywordModel. Do *not* rely on
          * the current book as given by the DhSidebar, as that will change
-         * whenever a hit is clicked. */
+         * whenever a hit is clicked.
+         */
         if (current_book_flag)
                 weight = PANGO_WEIGHT_BOLD;
         else
                 weight = PANGO_WEIGHT_NORMAL;
 
         link_type = dh_link_get_link_type (link);
-        if (link_type == DH_LINK_TYPE_STRUCT || link_type == DH_LINK_TYPE_PROPERTY ||
+
+        if (link_type == DH_LINK_TYPE_STRUCT ||
+            link_type == DH_LINK_TYPE_PROPERTY ||
             link_type == DH_LINK_TYPE_SIGNAL) {
-                name = g_markup_printf_escaped (
-                                "%s <i><small><span weight=\"normal\">(%s)</span></small></i>",
-                                dh_link_get_name (link),
-                                dh_link_type_to_string (dh_link_get_link_type (link)));
+                name = g_markup_printf_escaped ("%s <i><small><span weight=\"normal\">(%s)</span></small></i>",
+                                                dh_link_get_name (link),
+                                                dh_link_type_to_string (link_type));
         } else {
                 name = g_markup_printf_escaped ("%s", dh_link_get_name (link));
         }
@@ -453,108 +434,27 @@ hitlist_cell_data_func (GtkTreeViewColumn *tree_column,
                       "style", style,
                       "weight", weight,
                       NULL);
+
         g_free (name);
 }
 
 static void
-sidebar_book_tree_link_selected_cb (DhBookTree *book_tree,
-                                    DhLink     *link,
-                                    DhSidebar  *sidebar)
+book_tree_link_selected_cb (DhBookTree *book_tree,
+                            DhLink     *link,
+                            DhSidebar  *sidebar)
 {
-        g_signal_emit (sidebar, signals[LINK_SELECTED], 0, link);
-}
-
-/**
- * dh_sidebar_get_selected_book:
- * @sidebar: a #DhSidebar.
- *
- * Returns: (nullable) (transfer full): the #DhLink of the selected book, or
- * %NULL if there is no selection. Unref with dh_link_unref() when no longer
- * needed.
- */
-DhLink *
-dh_sidebar_get_selected_book (DhSidebar *sidebar)
-{
-        DhSidebarPrivate *priv;
-
-        g_return_val_if_fail (DH_IS_SIDEBAR (sidebar), NULL);
-
-        priv = dh_sidebar_get_instance_private (sidebar);
-
-        return dh_book_tree_get_selected_book (priv->book_tree);
-}
-
-/**
- * dh_sidebar_select_uri:
- * @sidebar: a #DhSidebar.
- * @uri: the URI to select.
- */
-void
-dh_sidebar_select_uri (DhSidebar   *sidebar,
-                       const gchar *uri)
-{
-        DhSidebarPrivate *priv;
-
-        g_return_if_fail (DH_IS_SIDEBAR (sidebar));
-
-        priv = dh_sidebar_get_instance_private (sidebar);
-
-        dh_book_tree_select_uri (priv->book_tree, uri);
-}
-
-/**
- * dh_sidebar_new:
- * @book_manager: (nullable): a #DhBookManager. This parameter is deprecated,
- * you should just pass %NULL.
- *
- * Returns: (transfer floating): a new #DhSidebar widget.
- */
-GtkWidget *
-dh_sidebar_new (DhBookManager *book_manager)
-{
-        return GTK_WIDGET (g_object_new (DH_TYPE_SIDEBAR,
-                                         "orientation", GTK_ORIENTATION_VERTICAL,
-                                         NULL));
-}
-
-static void
-dh_sidebar_dispose (GObject *object)
-{
-        DhSidebarPrivate *priv = dh_sidebar_get_instance_private (DH_SIDEBAR (object));
-
-        g_clear_object (&priv->hitlist_model);
-
-        if (priv->idle_complete_id != 0) {
-                g_source_remove (priv->idle_complete_id);
-                priv->idle_complete_id = 0;
-        }
-
-        if (priv->idle_filter_id != 0) {
-                g_source_remove (priv->idle_filter_id);
-                priv->idle_filter_id = 0;
-        }
-
-        G_OBJECT_CLASS (dh_sidebar_parent_class)->dispose (object);
-}
-
-static void
-dh_sidebar_finalize (GObject *object)
-{
-        DhSidebarPrivate *priv = dh_sidebar_get_instance_private (DH_SIDEBAR (object));
-
-        g_clear_pointer (&priv->completion, g_completion_free);
-
-        G_OBJECT_CLASS (dh_sidebar_parent_class)->finalize (object);
+        g_signal_emit (sidebar, signals[SIGNAL_LINK_SELECTED], 0, link);
 }
 
 static void
 dh_sidebar_init (DhSidebar *sidebar)
 {
-        DhSidebarPrivate *priv;
+        DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
         GtkCellRenderer *cell;
         DhBookManager *book_manager;
 
-        priv = dh_sidebar_get_instance_private (sidebar);
+        gtk_orientable_set_orientation (GTK_ORIENTABLE (sidebar),
+                                        GTK_ORIENTATION_VERTICAL);
 
         /* Setup the search entry */
         priv->entry = GTK_ENTRY (gtk_search_entry_new ());
@@ -562,16 +462,26 @@ dh_sidebar_init (DhSidebar *sidebar)
         g_object_set (priv->entry,
                       "margin", 6,
                       NULL);
-        gtk_box_pack_start (GTK_BOX (sidebar), GTK_WIDGET (priv->entry), FALSE, FALSE, 0);
+        gtk_container_add (GTK_CONTAINER (sidebar), GTK_WIDGET (priv->entry));
 
-        g_signal_connect (priv->entry, "key-press-event",
-                          G_CALLBACK (sidebar_entry_key_press_event_cb),
+        g_signal_connect (priv->entry,
+                          "key-press-event",
+                          G_CALLBACK (entry_key_press_event_cb),
                           sidebar);
-        g_signal_connect (priv->entry, "changed",
-                          G_CALLBACK (sidebar_entry_changed_cb),
+
+        g_signal_connect (priv->entry,
+                          "changed",
+                          G_CALLBACK (entry_changed_cb),
                           sidebar);
-        g_signal_connect (priv->entry, "insert-text",
-                          G_CALLBACK (sidebar_entry_insert_text_cb),
+
+        g_signal_connect (priv->entry,
+                          "search-changed",
+                          G_CALLBACK (entry_search_changed_cb),
+                          sidebar);
+
+        g_signal_connect (priv->entry,
+                          "insert-text",
+                          G_CALLBACK (entry_insert_text_cb),
                           sidebar);
 
         /* Setup hitlist */
@@ -582,13 +492,14 @@ dh_sidebar_init (DhSidebar *sidebar)
         gtk_tree_view_set_enable_search (priv->hitlist_view, FALSE);
         gtk_widget_show (GTK_WIDGET (priv->hitlist_view));
 
-        g_signal_connect (priv->hitlist_view, "button-press-event",
-                          G_CALLBACK (sidebar_hitlist_button_press_cb),
+        g_signal_connect (priv->hitlist_view,
+                          "button-press-event",
+                          G_CALLBACK (hitlist_button_press_cb),
                           sidebar);
 
         g_signal_connect (gtk_tree_view_get_selection (priv->hitlist_view),
                           "changed",
-                          G_CALLBACK (sidebar_hitlist_selection_changed_cb),
+                          G_CALLBACK (hitlist_selection_changed_cb),
                           sidebar);
 
         cell = gtk_cell_renderer_text_new ();
@@ -611,34 +522,36 @@ dh_sidebar_init (DhSidebar *sidebar)
                                         GTK_POLICY_AUTOMATIC);
         gtk_container_add (GTK_CONTAINER (priv->sw_hitlist),
                            GTK_WIDGET (priv->hitlist_view));
-        gtk_box_pack_start (GTK_BOX (sidebar), GTK_WIDGET (priv->sw_hitlist), TRUE, TRUE, 0);
+        gtk_widget_set_hexpand (GTK_WIDGET (priv->sw_hitlist), TRUE);
+        gtk_widget_set_vexpand (GTK_WIDGET (priv->sw_hitlist), TRUE);
+        gtk_container_add (GTK_CONTAINER (sidebar), GTK_WIDGET (priv->sw_hitlist));
 
         /* Setup book manager */
         book_manager = dh_book_manager_get_singleton ();
 
         g_signal_connect_object (book_manager,
                                  "book-created",
-                                 G_CALLBACK (sidebar_book_created_or_enabled_cb),
+                                 G_CALLBACK (book_manager_changed_cb),
                                  sidebar,
-                                 0);
+                                 G_CONNECT_SWAPPED);
 
         g_signal_connect_object (book_manager,
                                  "book-enabled",
-                                 G_CALLBACK (sidebar_book_created_or_enabled_cb),
+                                 G_CALLBACK (book_manager_changed_cb),
                                  sidebar,
-                                 0);
+                                 G_CONNECT_SWAPPED);
 
         g_signal_connect_object (book_manager,
                                  "book-deleted",
-                                 G_CALLBACK (sidebar_book_deleted_or_disabled_cb),
+                                 G_CALLBACK (book_manager_changed_cb),
                                  sidebar,
-                                 0);
+                                 G_CONNECT_SWAPPED);
 
         g_signal_connect_object (book_manager,
                                  "book-disabled",
-                                 G_CALLBACK (sidebar_book_deleted_or_disabled_cb),
+                                 G_CALLBACK (book_manager_changed_cb),
                                  sidebar,
-                                 0);
+                                 G_CONNECT_SWAPPED);
 
         /* Setup the book tree */
         priv->sw_book_tree = GTK_SCROLLED_WINDOW (gtk_scrolled_window_new (NULL, NULL));
@@ -652,35 +565,94 @@ dh_sidebar_init (DhSidebar *sidebar)
         gtk_widget_show (GTK_WIDGET (priv->book_tree));
         g_signal_connect (priv->book_tree,
                           "link-selected",
-                          G_CALLBACK (sidebar_book_tree_link_selected_cb),
+                          G_CALLBACK (book_tree_link_selected_cb),
                           sidebar);
         gtk_container_add (GTK_CONTAINER (priv->sw_book_tree), GTK_WIDGET (priv->book_tree));
-        gtk_box_pack_end (GTK_BOX (sidebar), GTK_WIDGET (priv->sw_book_tree), TRUE, TRUE, 0);
-
-        sidebar_completion_populate (sidebar);
+        gtk_widget_set_hexpand (GTK_WIDGET (priv->sw_book_tree), TRUE);
+        gtk_widget_set_vexpand (GTK_WIDGET (priv->sw_book_tree), TRUE);
+        gtk_container_add (GTK_CONTAINER (sidebar), GTK_WIDGET (priv->sw_book_tree));
 
         gtk_widget_show_all (GTK_WIDGET (sidebar));
 }
 
-static void
-dh_sidebar_class_init (DhSidebarClass *klass)
+/**
+ * dh_sidebar_new:
+ * @book_manager: (nullable): a #DhBookManager. This parameter is deprecated,
+ * you should just pass %NULL.
+ *
+ * Returns: (transfer floating): a new #DhSidebar widget.
+ */
+GtkWidget *
+dh_sidebar_new (DhBookManager *book_manager)
 {
-        GObjectClass *object_class = G_OBJECT_CLASS (klass);
+        return g_object_new (DH_TYPE_SIDEBAR, NULL);
+}
 
-        object_class->dispose = dh_sidebar_dispose;
-        object_class->finalize = dh_sidebar_finalize;
+/**
+ * dh_sidebar_select_uri:
+ * @sidebar: a #DhSidebar.
+ * @uri: the URI to select.
+ */
+void
+dh_sidebar_select_uri (DhSidebar   *sidebar,
+                       const gchar *uri)
+{
+        DhSidebarPrivate *priv;
 
-        /**
-         * DhSidebar::link-selected:
-         * @sidebar: a #DhSidebar.
-         * @link: the selected #DhLink.
+        g_return_if_fail (DH_IS_SIDEBAR (sidebar));
+        g_return_if_fail (uri != NULL);
+
+        priv = dh_sidebar_get_instance_private (sidebar);
+
+        dh_book_tree_select_uri (priv->book_tree, uri);
+}
+
+/**
+ * dh_sidebar_set_search_string:
+ * @sidebar: a #DhSidebar.
+ * @str: the string to search.
+ */
+void
+dh_sidebar_set_search_string (DhSidebar   *sidebar,
+                              const gchar *str)
+{
+        DhSidebarPrivate *priv;
+
+        g_return_if_fail (DH_IS_SIDEBAR (sidebar));
+        g_return_if_fail (str != NULL);
+
+        priv = dh_sidebar_get_instance_private (sidebar);
+
+        gtk_entry_set_text (priv->entry, str);
+        gtk_editable_select_region (GTK_EDITABLE (priv->entry), 0, 0);
+        gtk_editable_set_position (GTK_EDITABLE (priv->entry), -1);
+
+        /* If the GtkEntry text was already equal to @str, the
+         * GtkEditable::changed signal was not emitted, so force to emit it to
+         * call entry_changed_cb() and entry_search_changed_cb(), forcing a new
+         * search. If an exact match is found, the DhSidebar::link-selected
+         * signal will be emitted, to re-jump to that symbol (even if the
+         * GtkEntry text was equal, it doesn't mean that the WebKitWebView was
+         * showing the exact match).
+         * https://bugzilla.gnome.org/show_bug.cgi?id=776596
          */
-        signals[LINK_SELECTED] =
-                g_signal_new ("link-selected",
-                              G_TYPE_FROM_CLASS (klass),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (DhSidebarClass, link_selected),
-                              NULL, NULL, NULL,
-                              G_TYPE_NONE,
-                              1, DH_TYPE_LINK);
+        g_signal_emit_by_name (priv->entry, "changed");
+}
+
+/**
+ * dh_sidebar_set_search_focus:
+ * @sidebar: a #DhSidebar.
+ *
+ * Gives the focus to the search entry.
+ */
+void
+dh_sidebar_set_search_focus (DhSidebar *sidebar)
+{
+        DhSidebarPrivate *priv;
+
+        g_return_if_fail (DH_IS_SIDEBAR (sidebar));
+
+        priv = dh_sidebar_get_instance_private (sidebar);
+
+        gtk_widget_grab_focus (GTK_WIDGET (priv->entry));
 }
