@@ -14,25 +14,19 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "config.h"
 #include "dh-window.h"
-
-#include <string.h>
 #include <math.h>
-#include <glib/gi18n-lib.h>
-#include <gdk/gdkkeysyms.h>
+#include <glib/gi18n.h>
 #include <webkit2/webkit2.h>
-
-#include "dh-book-manager.h"
 #include "dh-book.h"
+#include "dh-book-manager.h"
+#include "dh-settings.h"
 #include "dh-sidebar.h"
 #include "dh-util.h"
-#include "dh-settings.h"
-#include "dh-enum-types-app.h"
 
 typedef struct {
         GMenuModel     *gear_app_menu;
@@ -48,18 +42,9 @@ typedef struct {
         GtkButton      *go_down_button;
 
         DhLink         *selected_search_link;
-        GtkSettings    *gtk_settings;
         guint           configure_id;
-        gulong          xft_dpi_changed_id;
 } DhWindowPrivate;
 
-enum {
-        OPEN_LINK,
-        N_SIGNALS
-};
-
-static guint signals[N_SIGNALS] = { 0 };
-
 #define WINDOW_SETTINGS_SAVE_TIMEOUT_MSECS 100
 
 static guint tab_accel_keys[] = {
@@ -505,25 +490,6 @@ gear_menu_cb (GSimpleAction *action,
         g_variant_unref (state);
 }
 
-static void
-dh_window_open_link (DhWindow        *window,
-                     const char      *location,
-                     DhOpenLinkFlags  flags)
-{
-        switch (flags) {
-                case DH_OPEN_LINK_NEW_WINDOW:
-                        dh_app_new_window (DH_APP (gtk_window_get_application (GTK_WINDOW (window))));
-                        break;
-
-                case DH_OPEN_LINK_NEW_TAB:
-                        window_open_new_tab (window, location, FALSE);
-                        break;
-
-                default:
-                        g_assert_not_reached ();
-        }
-}
-
 static GActionEntry win_entries[] = {
         /* tabs */
         { "new-tab",          new_tab_cb },
@@ -574,86 +540,6 @@ settings_fonts_changed_cb (DhSettings  *settings,
         }
 }
 
-static void
-update_fonts_on_dpi_change (DhWindow *window)
-{
-        DhWindowPrivate *priv;
-        DhSettings *settings;
-        gchar *font_fixed = NULL;
-        gchar *font_variable = NULL;
-        WebKitWebView *view;
-        gint i;
-
-        priv = dh_window_get_instance_private (window);
-
-        settings = dh_settings_get_singleton ();
-        dh_settings_get_selected_fonts (settings, &font_fixed, &font_variable);
-
-        if (font_fixed != NULL && font_variable != NULL) {
-                /* change font for all pages */
-                for (i = 0; i < gtk_notebook_get_n_pages (priv->notebook); i++) {
-                        GtkWidget *page = gtk_notebook_get_nth_page (priv->notebook, i);
-                        view = WEBKIT_WEB_VIEW (g_object_get_data (G_OBJECT (page), "web_view"));
-                        dh_util_view_set_font (view, font_fixed, font_variable);
-                }
-        }
-
-        g_free (font_fixed);
-        g_free (font_variable);
-}
-
-static void
-gtk_xft_dpi_changed_cb (GtkSettings *gtk_settings,
-                        GParamSpec  *pspec,
-                        gpointer     user_data)
-{
-        DhWindow *window = DH_WINDOW (user_data);
-        update_fonts_on_dpi_change (window);
-}
-
-/* Monitor GdkScreen and GtkSettings for DPI changes. */
-static void
-dh_window_screen_changed (GtkWidget *window,
-                          GdkScreen *previous_screen)
-{
-        DhWindow *dh_window = DH_WINDOW(window);
-        DhWindowPrivate *priv = dh_window_get_instance_private (dh_window);
-        GtkSettings *previous_settings = gtk_settings_get_for_screen (previous_screen);
-        GtkSettings *current_settings = gtk_widget_get_settings (window);
-
-        /* If the screen has changed we need to re-retrieve the GtkSettings object,
-         * and disconnect the old signal handlers before re-connecting again.
-         */
-        if (current_settings != previous_settings) {
-                if (priv->xft_dpi_changed_id != 0) {
-                        if (priv->gtk_settings != NULL &&
-                            g_signal_handler_is_connected (priv->gtk_settings, priv->xft_dpi_changed_id))
-                                g_signal_handler_disconnect (priv->gtk_settings, priv->xft_dpi_changed_id);
-                        priv->xft_dpi_changed_id = 0;
-                }
-
-                priv->gtk_settings = NULL;
-        }
-
-        /* Now store the new GtkSettings and (re)connect the signals if needed. */
-        if (priv->gtk_settings == NULL) {
-                GdkScreen *screen = gtk_widget_get_screen (window);
-                priv->gtk_settings = gtk_settings_get_for_screen (screen);
-                priv->xft_dpi_changed_id =
-                        g_signal_connect (priv->gtk_settings,
-                                          "notify::gtk-xft-dpi",
-                                          G_CALLBACK (gtk_xft_dpi_changed_cb),
-                                          window);
-
-        }
-
-        update_fonts_on_dpi_change (dh_window);
-
-        if (GTK_WIDGET_CLASS (dh_window_parent_class)->screen_changed != NULL)
-                GTK_WIDGET_CLASS (dh_window_parent_class)->screen_changed (window,
-                                                                           previous_screen);
-}
-
 /* FIXME: this is a bit complicated just for saving the window state.
  * Implement something (hopefully) simpler, by saving the GSettings only when
  * needed:
@@ -677,8 +563,7 @@ window_settings_save_cb (gpointer user_data)
 
         settings = dh_settings_get_singleton ();
         dh_util_window_settings_save (GTK_WINDOW (window),
-                                      dh_settings_peek_window_settings (settings),
-                                      TRUE);
+                                      dh_settings_peek_window_settings (settings));
 
         return G_SOURCE_REMOVE;
 }
@@ -724,8 +609,7 @@ dh_window_delete_event (GtkWidget   *widget,
 
         settings = dh_settings_get_singleton ();
         dh_util_window_settings_save (GTK_WINDOW (window),
-                                      dh_settings_peek_window_settings (settings),
-                                      TRUE);
+                                      dh_settings_peek_window_settings (settings));
 
         if (GTK_WIDGET_CLASS (dh_window_parent_class)->delete_event == NULL)
                 return GDK_EVENT_PROPAGATE;
@@ -737,6 +621,7 @@ static void
 dh_window_init (DhWindow *window)
 {
         DhWindowPrivate *priv;
+        GtkApplication *app;
         DhSettings    *settings;
         GtkAccelGroup *accel_group;
         GClosure      *closure;
@@ -747,7 +632,8 @@ dh_window_init (DhWindow *window)
         priv = dh_window_get_instance_private (window);
         priv->selected_search_link = NULL;
 
-        if (!_dh_app_has_app_menu (DH_APP (g_application_get_default ()))) {
+        app = GTK_APPLICATION (g_application_get_default ());
+        if (!gtk_application_prefers_app_menu (app)) {
                 gtk_menu_button_set_menu_model (priv->gear_menu_button, priv->gear_app_menu);
         }
 
@@ -759,13 +645,6 @@ dh_window_init (DhWindow *window)
                                  window,
                                  0);
 
-        /* we can't get the GdkScreen for the widget here, so get the
-         * GtkSettings associated to the default one instead for now */
-        priv->gtk_settings = gtk_settings_get_default ();
-        priv->xft_dpi_changed_id =
-                g_signal_connect (priv->gtk_settings, "notify::gtk-xft-dpi",
-                                  G_CALLBACK (gtk_xft_dpi_changed_cb), window);
-
         g_action_map_add_action_entries (G_ACTION_MAP (window),
                                          win_entries, G_N_ELEMENTS (win_entries),
                                          window);
@@ -797,13 +676,6 @@ dh_window_dispose (GObject *object)
                 priv->configure_id = 0;
         }
 
-        if (priv->xft_dpi_changed_id != 0) {
-                if (priv->gtk_settings != NULL &&
-                    g_signal_handler_is_connected (priv->gtk_settings, priv->xft_dpi_changed_id))
-                        g_signal_handler_disconnect (priv->gtk_settings, priv->xft_dpi_changed_id);
-                priv->xft_dpi_changed_id = 0;
-        }
-
         G_OBJECT_CLASS (dh_window_parent_class)->dispose (object);
 }
 
@@ -815,23 +687,9 @@ dh_window_class_init (DhWindowClass *klass)
 
         object_class->dispose = dh_window_dispose;
 
-        widget_class->screen_changed = dh_window_screen_changed;
         widget_class->configure_event = dh_window_configure_event;
         widget_class->delete_event = dh_window_delete_event;
 
-        klass->open_link = dh_window_open_link;
-
-        signals[OPEN_LINK] =
-                g_signal_new ("open-link",
-                              G_TYPE_FROM_CLASS (klass),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (DhWindowClass, open_link),
-                              NULL, NULL, NULL,
-                              G_TYPE_NONE,
-                              2,
-                              G_TYPE_STRING,
-                              DH_TYPE_OPEN_LINK_FLAGS);
-
         /* Bind class to template */
         gtk_widget_class_set_template_from_resource (widget_class,
                                                      "/org/gnome/devhelp/dh-window.ui");
@@ -1039,7 +897,7 @@ window_web_view_decide_policy_cb (WebKitWebView            *web_view,
         state = webkit_navigation_action_get_modifiers (navigation_action);
         if (button == 2 || (button == 1 && state == GDK_CONTROL_MASK)) {
                 webkit_policy_decision_ignore (policy_decision);
-                g_signal_emit (window, signals[OPEN_LINK], 0, uri, DH_OPEN_LINK_NEW_TAB);
+                window_open_new_tab (window, uri, FALSE);
                 return GDK_EVENT_STOP;
         }
 
@@ -1217,9 +1075,6 @@ window_web_view_button_press_event_cb (WebKitWebView  *web_view,
                                        DhWindow       *window)
 {
         switch (event->button) {
-                case 3:
-                        return GDK_EVENT_STOP;
-
                 /* Some mice emit button presses when the scroll wheel is tilted
                  * to the side. Web browsers use them to navigate in history.
                  */
@@ -1642,12 +1497,14 @@ window_tab_set_title (DhWindow      *window,
 }
 
 GtkWidget *
-dh_window_new (DhApp *application)
+dh_window_new (GtkApplication *application)
 {
         DhWindow *window;
         DhWindowPrivate *priv;
         DhSettings *settings;
 
+        g_return_val_if_fail (GTK_IS_APPLICATION (application), NULL);
+
         window = g_object_new (DH_TYPE_WINDOW,
                                "application", application,
                                NULL);
@@ -1657,9 +1514,9 @@ dh_window_new (DhApp *application)
         window_populate (window);
 
         settings = dh_settings_get_singleton ();
+        gtk_widget_realize (GTK_WIDGET (window));
         dh_util_window_settings_restore (GTK_WINDOW (window),
-                                         dh_settings_peek_window_settings (settings),
-                                         TRUE);
+                                         dh_settings_peek_window_settings (settings));
 
         g_settings_bind (dh_settings_peek_paned_settings (settings), "position",
                          priv->hpaned, "position",
